globals
[
  left-edge  ;; the pxcor of the left margin
  right-edge  ;; the pxcor of the right margin
  text-width  ;; the current maximum number of characters any row of text can have
  text  ;; the text to be written
]

turtles-own
[ 
  word-length  ;; the length of the word the turtle is in
  num-word  ;; the number the turtle's word is in the text
  my-leader  ;; the turtle with the character that comes before this turtle's character
  new-line?  ;; whether we should start a new line or not
]

;; setup the variables and the Graphics Window
to setup
  ca
  cc
  
  ;; these are initial settings for the margin locations
  set left-edge round ( 0 - screen-edge-x ) / 2 - 1
  set right-edge round screen-edge-x / 2 + 2
  ;; color the margins red
  ask patches with [ pxcor = left-edge or pxcor = right-edge ]
  [ set pcolor red ]
  set text-width right-edge - left-edge - 1
  
  ;; set the text to be displayed
  set text "These are times which try men's souls"
  
  ;; create enough turtles to have 1 turtle for every character in text
  cct length text
  [
    set xcor left-edge + ( who mod text-width ) + 1
    set ycor screen-edge-y - ( ( int ( who / text-width ) ) * line-spacing )
    set word-length 0
    set my-leader nobody
    set new-line? false
    ;; make the turtle "disappear".  we can't use hide-turtle
    ;; since we want to be able to see the turtle's label
    set color pcolor
  ]
  setup-letters
end

;; assign the letters to the turtles, determine who leads them,
;; and set how long each word is
to setup-letters
  locals
  [
    remaining-text
    word-count
    prev-letter
    turtles-in-word
    index
  ]
  
  set remaining-text text
  set word-count 1
  set prev-letter nobody
  
  ask turtles
  [
    without-interruption
    [
      set label first remaining-text
      set my-leader prev-letter
      set num-word word-count
      if label-is-whitespace?
      [
        set word-count word-count + 2
        set num-word num-word + 1
      ]
      set remaining-text but-first remaining-text
      set prev-letter self
    ]
  ]
  
  set index 1
  repeat word-count
  [
    set turtles-in-word turtles with [ num-word = index ]
    ask turtles-in-word
    [ set word-length count turtles-in-word ]
    set index index + 1
  ]
end

;; report true if the turtle's label is a space or a tab
to-report label-is-whitespace?  ;; turtle procedure
  report label = " " or label = "\t"
end

;; the main procedure called by the GO forever button
to go
  ask turtles
  [
    set new-line? false
    
    pick-heading  ;; choose the heading to the appropriate patch
    
    ;; if the turtle is not satisfied with where it is in 
    ;; relation to its leader, go forward 1
    if not happy?  
    [ fd  1 ]
    update-color
  ]
end

;; set the heading of the turtle to the appropriate patch
to pick-heading
  ifelse my-leader = nobody
  [ set heading towards-nowrap patch ( left-edge + 1 ) screen-edge-y ]  ;; the first letter needs to go to the upper-left-most patch
  [
    set heading towards-nowrap value-from my-leader [ patch-at 1 0 ]  ;; set the heading to the patch to the right of the leader's patch
    ifelse( text-width < word-length )
    [
      ;; if our word is too long for the width of text,
      ;; check to see if the patch to the right of the leader is on or over the right margin
      if ( ( value-from my-leader [ pxcor ] ) + 1 ) >= ( right-edge )
      [
        ;; if it is change the heading to the beginning of the next line
        set new-line? true
        set heading towards-nowrap patch ( left-edge + 1 ) new-line-pycor
      ]
    ]
    [
      ;; if our word is short enough to fit on a single line but there is not
      ;; enough room on this particular line, try to word wrap.  that is to say,
      ;; if the turtle is at the beginning of a word and the word is too long to
      ;; be completed on a single line given the starting point of the word, 
      ;; have the leader of the word, move to the next line.
      if ( num-word != num-word-of my-leader ) and ( ( value-from my-leader [ pxcor ] ) + word-length + 1 ) > ( right-edge )
      [
        ;; if it is change the heading to the beginning of the next line
        set new-line? true
        set heading towards-nowrap patch ( left-edge + 1 ) new-line-pycor
      ]
    ]
  ]
end

;; reports pycor of the new line the turtle is supposed to go to
;; based on the location of its leader and the width of the text
to-report new-line-pycor  ;; turtle procedure
  if abs ( ( value-from my-leader [pycor] ) - line-spacing ) > screen-edge-y
  [ report value-from my-leader [ pycor ] ]
  report ( ( value-from my-leader [pycor] ) - line-spacing )
end

;; reports true if turtle is satisifed with its current position
to-report happy?  ;; turtle procedure
  if my-leader = nobody  ;; if the turtle is the first letter, is it on the upper-left-most patch that it can be?
  [ report ( pxcor = ( left-edge + 1 ) ) and ( pycor = screen-edge-y ) ]  
  ifelse new-line?  ;; do we want to start a new-line?
  [ report ( pxcor = left-edge + 1 ) and ( pycor = new-line-pycor ) ]  ;; is the turtle at the beginning of the next line?
  [ report ( patch-at -1 0 = value-from my-leader [ patch-here ] ) ]  ;; is the turtle on the patch to the right of its leader?
end

;; set the turtle's color to the patch's color to make the turtle "disappear"
to update-color  ;; turtle procedure
  if pcolor != color
  [ set color pcolor ]
end

;; scatter the letters
to scatter  ;; turtle procedure
  fd random 100
  update-color
end

;; these procedures allow you to move the margins with the mouse
to move-margins
  if ( mouse-down? )
  [
    if ( round mouse-xcor = left-edge )
    [ move-one-margin true ]
    if ( round mouse-xcor = right-edge )
    [ move-one-margin false ]
    set text-width right-edge - left-edge - 1  ;; recompute the proper width for the text
  ]
end

;; move one marging to a new location
to move-one-margin [ moving-left-edge? ]
  locals
  [
    old-edge  ;; the old pxcor of the moving-edge margin
    moving-edge  ;; the pxcor of the edge that is moving
  ]
  
  if mouse-down?
  [
    if valid-move? round mouse-xcor moving-left-edge?
    [
      ifelse moving-left-edge?
      [ set moving-edge left-edge ]
      [ set moving-edge right-edge ]
      set old-edge moving-edge

      ;; update the value of the edge that we are moving
      set moving-edge round mouse-xcor
      ifelse moving-left-edge?
      [ set left-edge moving-edge ]
      [ set right-edge moving-edge ]
      
      ;; ask the patches from the old and new margins, 
      ;; and any turtles on them to update themselves to the proper color
      ask patches with [ pxcor = moving-edge or pxcor = old-edge ]
      [
        show-margin moving-edge
        ask turtles-here
        [ update-color ]
      ]
    ]
    move-one-margin moving-left-edge?  ;; let's try moving the margin again
  ]
end

;; report true if we can move to the edge to this pxcor.
;; we always want to have at least 1 patch in between the margins
to-report valid-move? [ potential-edge moving-left-edge? ]
  ifelse moving-left-edge?
  [ report round mouse-xcor < right-edge - 1 ]
  [ report round mouse-xcor > left-edge + 1 ]
end

;; update the location of the margin and remove the old margin.
;; be sure to call this only on the patches that are for the currently moving margin.
;; if you don't, you could end up with only 1 margin!
to show-margin [ edge ]  ;; patch procedure
  ifelse pxcor = edge and pcolor != red
  [ set pcolor red ]
  [
    if pxcor != edge and pcolor != black
    [ set pcolor black ]
  ]
end


; *** NetLogo Model Copyright Notice ***
;
; This model was created as part of the project: CONNECTED MATHEMATICS:
; MAKING SENSE OF COMPLEX PHENOMENA THROUGH BUILDING OBJECT-BASED PARALLEL
; MODELS (OBPML).  The project gratefully acknowledges the support of the
; National Science Foundation (Applications of Advanced Technologies
; Program) -- grant numbers RED #9552950 and REC #9632612.
;
; Copyright 1998 by Uri Wilensky. All rights reserved.
;
; Permission to use, modify or redistribute this model is hereby granted,
; provided that both of the following requirements are followed:
; a) this copyright notice is included.
; b) this model will not be redistributed for profit without permission
;    from Uri Wilensky.
; Contact Uri Wilensky for appropriate licenses for redistribution for
; profit.
;
; This model was converted to NetLogo as part of the project:
; PARTICIPATORY SIMULATIONS: NETWORK-BASED DESIGN FOR SYSTEMS LEARNING IN
; CLASSROOMS.  The project gratefully acknowledges the support of the
; National Science Foundation (REPP program) -- grant number REC #9814682.
; Converted from StarLogoT to NetLogo, 2001.  Updated 2002.
;
; To refer to this model in academic publications, please use:
; Wilensky, U. (1998).  NetLogo Wandering Letters model.
; http://ccl.northwestern.edu/netlogo/models/WanderingLetters.
; Center for Connected Learning and Computer-Based Modeling,
; Northwestern University, Evanston, IL.
;
; *** End of NetLogo Model Copyright Notice ***
@#$#@#$#@
GRAPHICS-WINDOW
258
10
645
418
14
14
13.0
0
10
0
0

CC-WINDOW
3
239
252
417
Command Center

SLIDER
3
90
120
123
line-spacing
line-spacing
1
3
3
1
1
NIL

BUTTON
72
41
195
74
Scatter Letters
scatter
NIL
1
T
TURTLE

BUTTON
196
41
256
74
Go
go
T
1
T
OBSERVER

BUTTON
2
41
71
74
Setup
setup
NIL
1
T
OBSERVER

MONITOR
172
174
257
223
Right Margin
right-edge
3
1

MONITOR
84
174
171
223
Mouse Xcor
mouse-xcor
1
1

MONITOR
1
174
83
223
Left Margin
left-edge
3
1

BUTTON
135
90
250
123
Move Margins
move-margins
T
1
T
OBSERVER

@#$#@#$#@
WHAT IS IT?
-----------
This model illustrates how to build a word processor where each of the letters acts independently.  Each letter knows only which letter comes before it and how long its word is.  When the letters or margins are moved, the letters find their way back to their proper locations.

HOW IT WORKS
------------
Each letter looks both for changes in the margins and for being out of sync with its "leader", which is the letter before it in the phrase.  If it is not in its proper place relative to its leader, it takes a step in its leader's direction.


HOW TO USE IT?
--------------
SETUP initializes the margins and a sample sentence for the text.

GO starts and stops the simulation.  

MOVE MARGINS allows you to click on a margin (a red line in the Graphics Window) and drag it to a new position.  If GO is pressed, the letters will automatically adjust to the new margins.

SCATTER LETTERS scatters the letters randomly about the Graphics Window.

LINE-SPACING controls how much space there is between lines.

SIMULATION-SPEED allows you to adjust the speed of the simulation.

LEFT MARGIN and RIGHT MARGIN display the locations of the left and right margins.

MOUSE XCOR displays the x-coordinate of the mouse while it is on the Graphics Window.


THINGS TO NOTICE
----------------
How do the letters find their way home?
In what ways is this different from the behavior of letters in a standard word processor?


THINGS TO TRY
-------------
While GO is pressed try:
--Pressing MOVE MARGINS and dragging the margins
--Changing the spacing
--Pushing the scatter button
--Combinations of all three


EXTENDING THE MODEL
-------------------
Can you extend the model so the user can input his/her own message?  You might want to use the user-input primitive for this.

Right now spaces are thought of like any other letter.  This means that sometimes a space will be at the beginning of a new line.  Try to change the model so that spaces are ignored at the ends of lines.


NETLOGO FEATURES
----------------
Note the use of mouse-down?, and mouse-xcor to move the margins. Both the procedure move-margins and the sub-procedures move-left-margin, move-right-margin recursively "listen" for user mouse clicks.


REFERENCES AND CREDITS
----------------------
This model is based on a Smalltalk program of the same name developed by Ted Kaehler of Disney Interactive.

To refer to this model in academic publications, please use: Wilensky, U. (1998).  NetLogo Wandering Letters model. http://ccl.northwestern.edu/netlogo/models/WanderingLetters. Center for Connected Learning and Computer-Based Modeling, Northwestern University, Evanston, IL.
@#$#@#$#@
default
true
0
Polygon -7566196 true true 150 5 40 250 150 205 260 250

ant
true
0
Polygon -7566196 true true 136 61 129 46 144 30 119 45 124 60 114 82 97 37 132 10 93 36 111 84 127 105 172 105 189 84 208 35 171 11 202 35 204 37 186 82 177 60 180 44 159 32 170 44 165 60
Polygon -7566196 true true 150 95 135 103 139 117 125 149 137 180 135 196 150 204 166 195 161 180 174 150 158 116 164 102
Polygon -7566196 true true 149 186 128 197 114 232 134 270 149 282 166 270 185 232 171 195 149 186 149 186
Polygon -7566196 true true 225 66 230 107 159 122 161 127 234 111 236 106
Polygon -7566196 true true 78 58 99 116 139 123 137 128 95 119
Polygon -7566196 true true 48 103 90 147 129 147 130 151 86 151
Polygon -7566196 true true 65 224 92 171 134 160 135 164 95 175
Polygon -7566196 true true 235 222 210 170 163 162 161 166 208 174
Polygon -7566196 true true 249 107 211 147 168 147 168 150 213 150
Polygon -7566196 true true 270 14
Polygon -7566196 true true 276 21

arrow
true
0
Polygon -7566196 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

bee
true
0
Polygon -256 true false 151 152 137 77 105 67 89 67 66 74 48 85 36 100 24 116 14 134 0 151 15 167 22 182 40 206 58 220 82 226 105 226 134 222
Polygon -16777216 true false 151 150 149 128 149 114 155 98 178 80 197 80 217 81 233 95 242 117 246 141 247 151 245 177 234 195 218 207 206 211 184 211 161 204 151 189 148 171
Polygon -7566196 true true 246 151 241 119 240 96 250 81 261 78 275 87 282 103 277 115 287 121 299 150 286 180 277 189 283 197 281 210 270 222 256 222 243 212 242 192
Polygon -16777216 true false 115 70 129 74 128 223 114 224
Polygon -16777216 true false 89 67 74 71 74 224 89 225 89 67
Polygon -16777216 true false 43 91 31 106 31 195 45 211
Line -1 false 200 144 213 70
Line -1 false 213 70 213 45
Line -1 false 214 45 203 26
Line -1 false 204 26 185 22
Line -1 false 185 22 170 25
Line -1 false 169 26 159 37
Line -1 false 159 37 156 55
Line -1 false 157 55 199 143
Line -1 false 200 141 162 227
Line -1 false 162 227 163 241
Line -1 false 163 241 171 249
Line -1 false 171 249 190 254
Line -1 false 192 253 203 248
Line -1 false 205 249 218 235
Line -1 false 218 235 200 144

bird1
false
0
Polygon -7566196 true true 2 6 2 39 270 298 297 298 299 271 187 160 279 75 276 22 100 67 31 0

bird2
false
0
Polygon -7566196 true true 2 4 33 4 298 270 298 298 272 298 155 184 117 289 61 295 61 105 0 43

boat1
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 33 230 157 182 150 169 151 157 156
Polygon -7566196 true true 149 55 88 143 103 139 111 136 117 139 126 145 130 147 139 147 146 146 149 55

boat2
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 157 54 175 79 174 96 185 102 178 112 194 124 196 131 190 139 192 146 211 151 216 154 157 154
Polygon -7566196 true true 150 74 146 91 139 99 143 114 141 123 137 126 131 129 132 139 142 136 126 142 119 147 148 147

boat3
false
0
Polygon -1 true false 63 162 90 207 223 207 290 162
Rectangle -6524078 true false 150 32 157 162
Polygon -16776961 true false 150 34 131 49 145 47 147 48 149 49
Polygon -7566196 true true 158 37 172 45 188 59 202 79 217 109 220 130 218 147 204 156 158 156 161 142 170 123 170 102 169 88 165 62
Polygon -7566196 true true 149 66 142 78 139 96 141 111 146 139 148 147 110 147 113 131 118 106 126 71

box
true
0
Polygon -7566196 true true 45 255 255 255 255 45 45 45

butterfly1
true
0
Polygon -16777216 true false 151 76 138 91 138 284 150 296 162 286 162 91
Polygon -7566196 true true 164 106 184 79 205 61 236 48 259 53 279 86 287 119 289 158 278 177 256 182 164 181
Polygon -7566196 true true 136 110 119 82 110 71 85 61 59 48 36 56 17 88 6 115 2 147 15 178 134 178
Polygon -7566196 true true 46 181 28 227 50 255 77 273 112 283 135 274 135 180
Polygon -7566196 true true 165 185 254 184 272 224 255 251 236 267 191 283 164 276
Line -7566196 true 167 47 159 82
Line -7566196 true 136 47 145 81
Circle -7566196 true true 165 45 8
Circle -7566196 true true 134 45 6
Circle -7566196 true true 133 44 7
Circle -7566196 true true 133 43 8

circle
false
0
Circle -7566196 true true 35 35 230

person
false
0
Circle -7566196 true true 155 20 63
Rectangle -7566196 true true 158 79 217 164
Polygon -7566196 true true 158 81 110 129 131 143 158 109 165 110
Polygon -7566196 true true 216 83 267 123 248 143 215 107
Polygon -7566196 true true 167 163 145 234 183 234 183 163
Polygon -7566196 true true 195 163 195 233 227 233 206 159

spacecraft
true
0
Polygon -7566196 true true 150 0 180 135 255 255 225 240 150 180 75 240 45 255 120 135

thin-arrow
true
0
Polygon -7566196 true true 150 0 0 150 120 150 120 293 180 293 180 150 300 150

truck-down
false
0
Polygon -7566196 true true 225 30 225 270 120 270 105 210 60 180 45 30 105 60 105 30
Polygon -8716033 true false 195 75 195 120 240 120 240 75
Polygon -8716033 true false 195 225 195 180 240 180 240 225

truck-left
false
0
Polygon -7566196 true true 120 135 225 135 225 210 75 210 75 165 105 165
Polygon -8716033 true false 90 210 105 225 120 210
Polygon -8716033 true false 180 210 195 225 210 210

truck-right
false
0
Polygon -7566196 true true 180 135 75 135 75 210 225 210 225 165 195 165
Polygon -8716033 true false 210 210 195 225 180 210
Polygon -8716033 true false 120 210 105 225 90 210

turtle
true
0
Polygon -7566196 true true 138 75 162 75 165 105 225 105 225 142 195 135 195 187 225 195 225 225 195 217 195 202 105 202 105 217 75 225 75 195 105 187 105 135 75 142 75 105 135 105

wolf-left
false
3
Polygon -6524078 true true 117 97 91 74 66 74 60 85 36 85 38 92 44 97 62 97 81 117 84 134 92 147 109 152 136 144 174 144 174 103 143 103 134 97
Polygon -6524078 true true 87 80 79 55 76 79
Polygon -6524078 true true 81 75 70 58 73 82
Polygon -6524078 true true 99 131 76 152 76 163 96 182 104 182 109 173 102 167 99 173 87 159 104 140
Polygon -6524078 true true 107 138 107 186 98 190 99 196 112 196 115 190
Polygon -6524078 true true 116 140 114 189 105 137
Rectangle -6524078 true true 109 150 114 192
Rectangle -6524078 true true 111 143 116 191
Polygon -6524078 true true 168 106 184 98 205 98 218 115 218 137 186 164 196 176 195 194 178 195 178 183 188 183 169 164 173 144
Polygon -6524078 true true 207 140 200 163 206 175 207 192 193 189 192 177 198 176 185 150
Polygon -6524078 true true 214 134 203 168 192 148
Polygon -6524078 true true 204 151 203 176 193 148
Polygon -6524078 true true 207 103 221 98 236 101 243 115 243 128 256 142 239 143 233 133 225 115 214 114

wolf-right
false
3
Polygon -6524078 true true 170 127 200 93 231 93 237 103 262 103 261 113 253 119 231 119 215 143 213 160 208 173 189 187 169 190 154 190 126 180 106 171 72 171 73 126 122 126 144 123 159 123
Polygon -6524078 true true 201 99 214 69 215 99
Polygon -6524078 true true 207 98 223 71 220 101
Polygon -6524078 true true 184 172 189 234 203 238 203 246 187 247 180 239 171 180
Polygon -6524078 true true 197 174 204 220 218 224 219 234 201 232 195 225 179 179
Polygon -6524078 true true 78 167 95 187 95 208 79 220 92 234 98 235 100 249 81 246 76 241 61 212 65 195 52 170 45 150 44 128 55 121 69 121 81 135
Polygon -6524078 true true 48 143 58 141
Polygon -6524078 true true 46 136 68 137
Polygon -6524078 true true 46 130
Polygon -6524078 true true 45 129 35 142 37 159 53 192 47 210 62 238 80 237
Line -16777216 false 74 237 59 213
Line -16777216 false 59 213 59 212
Line -16777216 false 58 211 67 192
Polygon -6524078 true true 38 138 66 149
Polygon -6524078 true true 46 128 33 120 21 118 11 123 3 138 5 160 13 178 9 192 0 199 20 196 25 179 24 161 25 148 45 140
Polygon -6524078 true true 67 122 96 126 63 144

@#$#@#$#@
NetLogo 1.3beta1
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
